unit fuAQBOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, Buttons, Spin, acQBBase, acAST,
  IniFiles;

type
  TfmAQBOptions = class(TForm)
    pnlBottom: TPanel;
	 pcOptions: TPageControl;
    tsCommon: TTabSheet;
    tsTree: TTabSheet;
    tsBuilder: TTabSheet;
    bCancel: TButton;
    bvBottom: TBevel;
    bOk: TButton;
    bSaveDefaults: TButton;
    pnlGroup3: TPanel;
    bvGroup3: TBevel;
    lbGroup3: TLabel;
	 pnlGroup1: TPanel;
    bvGroup1: TBevel;
    lbGroup1: TLabel;
	 pnlMainQuery: TPanel;
	 lbMainQuery: TLabel;
    bMainQuery: TButton;
    sbCopyMain: TSpeedButton;
	 sbPasteMain: TSpeedButton;
    pnlGroup2: TPanel;
	 bvGroup2: TBevel;
    lbGroup2: TLabel;
    pnlExpressionSubquery: TPanel;
    sbCopyExpression: TSpeedButton;
    sbPasteExpression: TSpeedButton;
    bExpressionSubquery: TButton;
    pnlDerivedSubquery: TPanel;
    sbCopyDerived: TSpeedButton;
    sbPasteDerived: TSpeedButton;
    bDerivedSubquery: TButton;
    lbExpressionSubquery: TLabel;
    lbDerivedSubquery: TLabel;
    pnlBuilderTop: TPanel;
    lbBuilderTop: TLabel;
	 gbTreeExpanding: TGroupBox;
    checkExpandFields: TCheckBox;
    checkExpandDatasources: TCheckBox;
    checkExpandSubqueries: TCheckBox;
    checkExpandUnions: TCheckBox;
    pnlTreeTop: TPanel;
	 lbTreeTop: TLabel;
	 gbTreeVisibility: TGroupBox;
    checkTreeShowFields: TCheckBox;
	 checkTreeShowObjects: TCheckBox;
	 gbTreeCaptions: TGroupBox;
	 editTreeFields: TEdit;
	 editTreeObjects: TEdit;
	 editTreeUnions: TEdit;
	 lbTreeExpanding: TLabel;
    lbTreeFields: TLabel;
    lbTreeObjects: TLabel;
    lbTreeUnions: TLabel;
	 pnlCommonTop: TPanel;
	 lbCommonTop: TLabel;
	 gbVisibility: TGroupBox;
	 checkShowTreePane: TCheckBox;
	 checkShowColumnsPane: TCheckBox;
	 lbSplitHorizontal: TLabel;
	 lbSplitVertical: TLabel;
	 editSplitVertical: TSpinEdit;
	 editSplitHorizontal: TSpinEdit;
	 gbAppearance: TGroupBox;
	 lbBorderWidth: TLabel;
	 editBorderWidth: TSpinEdit;
	 cbBkDarkness: TComboBox;
	 lbBorderStyle: TLabel;
	 lbBkDarkness: TLabel;
	 cbBorderStyle: TComboBox;
	 gbLayout: TGroupBox;
	 lbLinkPainter: TLabel;
	 cbLinkPainter: TComboBox;
	 lbAddObjectWindow: TLabel;
	 cbAddObjectWindow: TComboBox;
    pnlBuilderOptions: TPanel;
    editRightMargin: TSpinEdit;
    checkWordWrap: TCheckBox;
    lbRightMargin: TLabel;
	 procedure FormCreate(Sender: TObject);
	 procedure FormDestroy(Sender: TObject);
    procedure checkWordWrapClick(Sender: TObject);
	 procedure sbCopyMainClick(Sender: TObject);
	 procedure sbCopyExpressionClick(Sender: TObject);
    procedure sbCopyDerivedClick(Sender: TObject);
    procedure sbPasteMainClick(Sender: TObject);
    procedure sbPasteExpressionClick(Sender: TObject);
    procedure sbPasteDerivedClick(Sender: TObject);
    procedure bMainQueryClick(Sender: TObject);
    procedure bExpressionSubqueryClick(Sender: TObject);
    procedure bDerivedSubqueryClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure bSaveDefaultsClick(Sender: TObject);
  private
	 { Private declarations }
	 ClipSelectFormat: TSQLBuilderSelectFormat;
	 MainSelectFormat: TSQLBuilderSelectFormat;
	 ExpressionSelectFormat: TSQLBuilderSelectFormat;
	 DerivedSelectFormat: TSQLBuilderSelectFormat;

	 procedure LoadSettingsFromObject;

	 procedure LoadSettingsFromIni;
	 procedure SaveSettingsToIni;

	 procedure LoadSelectFormat(IniSection: String;
		 SelectFormat: TSQLBuilderSelectFormat);
	 procedure SaveSelectFormat(IniSection: String;
		 SelectFormat: TSQLBuilderSelectFormat);

	 procedure DisablePasteButtons;
	 procedure EnablePasteButtons;
  public
	 { Public declarations }
	 Settings: TIniFile;
	 QueryBuilder: TacQueryBuilder;
	 SQLBuilder: TacBaseSQLBuilder;

	 procedure SaveSettingsToObject;
  end;

var
  fmAQBOptions: TfmAQBOptions;

implementation

{$R *.dfm}

uses fuSQLBuilderOptions;

{ TfmAQBOptions }

procedure TfmAQBOptions.LoadSettingsFromObject;
begin
	with QueryBuilder do begin

		// Common options
		checkShowTreePane.Checked := TreeOptions.TreeVisible;
		checkShowColumnsPane.Checked := ColumnsPaneVisible;
		editSplitVertical.Value := TreeSplitterWidth;
		editSplitHorizontal.Value := SplitterHeight;

		editBorderWidth.Value := BorderWidth;
		cbBorderStyle.ItemIndex := Integer(BorderStyle);
		cbBkDarkness.ItemIndex := Integer(BkImageDarkness);

		cbAddObjectWindow.ItemIndex := Integer(AddObjectFormOptions.ListViewStyle);
		if LinkPainterClass = TacQueryBuilderLinkPainterSimple
			then cbLinkPainter.ItemIndex := 0;
		if LinkPainterClass = TacQueryBuilderLinkPainterAccess
			then cbLinkPainter.ItemIndex := 1;
		if LinkPainterClass = TacQueryBuilderLinkPainterMSSQL
			then cbLinkPainter.ItemIndex := 2;
	end;

	with QueryBuilder.TreeOptions do begin

		// Tree options
		checkTreeShowFields.Checked := ShowFields;
		checkTreeShowObjects.Checked := ShowFrom;

		editTreeFields.Text := FieldsNodeText;
		editTreeObjects.Text := FromNodeText;
		editTreeUnions.Text := UnionsNodeText;

		checkExpandFields.Checked := ExpandFieldsNodes;
		checkExpandDatasources.Checked := ExpandFromNodes;
		checkExpandSubqueries.Checked := ExpandQueryNodes;
		checkExpandUnions.Checked := ExpandUnionsNodes;
	end;

	with SQLBuilder do begin

		// SQL Builder options
		checkWordWrap.Checked := RightMargin = 0;
		checkWordWrapClick(nil);
		if RightMargin = 0
			then editRightMargin.Value := 80
			else editRightMargin.Value := RightMargin;

		MainSelectFormat.Assign(MainQueryFormat);
		ExpressionSelectFormat.Assign(ExpressionsSubQueryFormat);
		DerivedSelectFormat.Assign(FromSubQueryFormat);
	end;
end;

procedure TfmAQBOptions.SaveSettingsToObject;
begin
	with QueryBuilder do begin

		// Common options
		TreeOptions.TreeVisible := checkShowTreePane.Checked;
		ColumnsPaneVisible := checkShowColumnsPane.Checked;
		TreeSplitterWidth := editSplitVertical.Value;
		SplitterHeight := editSplitHorizontal.Value;

		BorderWidth := editBorderWidth.Value;
		BorderStyle := TacBorderStyle(cbBorderStyle.ItemIndex);

		// This option works only for Free Edition of Active Query Builder
		BkImageDarkness := TacBkImageDarkness(cbBkDarkness.ItemIndex);

		AddObjectFormOptions.ListViewStyle := TViewStyle(cbAddObjectWindow.ItemIndex);
		if cbLinkPainter.ItemIndex = 0
			then LinkPainterClass := TacQueryBuilderLinkPainterSimple;
		if cbLinkPainter.ItemIndex = 1
			then LinkPainterClass := TacQueryBuilderLinkPainterAccess;
		if cbLinkPainter.ItemIndex = 2
			then LinkPainterClass := TacQueryBuilderLinkPainterMSSQL;
	end;

	with QueryBuilder.TreeOptions do begin

		// Tree options
		ShowFields := checkTreeShowFields.Checked;
		ShowFrom := checkTreeShowObjects.Checked;

		FieldsNodeText := editTreeFields.Text;
		FromNodeText := editTreeObjects.Text;
		UnionsNodeText := editTreeUnions.Text;

		ExpandFieldsNodes := checkExpandFields.Checked;
		ExpandFromNodes := checkExpandDatasources.Checked;
		ExpandQueryNodes := checkExpandSubqueries.Checked;
		ExpandUnionsNodes := checkExpandUnions.Checked;
	end;

	with SQLBuilder do begin

		// SQL Builder options
		if checkWordWrap.Checked
			then RightMargin := 0
			else RightMargin := editRightMargin.Value;

		MainQueryFormat.Assign(MainSelectFormat);
		ExpressionsSubQueryFormat.Assign(ExpressionSelectFormat);
		FromSubQueryFormat.Assign(DerivedSelectFormat);
	end;
end;

procedure TfmAQBOptions.LoadSettingsFromIni;
begin
	with Settings do begin

		// Common options
		checkShowTreePane.Checked := ReadBool('Common', 'ShowTreePane', False);
		checkShowColumnsPane.Checked := ReadBool('Common', 'ShowColumnsPane', True);
		editSplitVertical.Value := ReadInteger('Common', 'SplitVertical', 2);
		editSplitHorizontal.Value := ReadInteger('Common', 'SplitHorizontal', 6);

		editBorderWidth.Value := ReadInteger('Common', 'BorderWidth', 3);
		cbBorderStyle.ItemIndex := ReadInteger('Common', 'BorderStyle', Integer(qbbs3D));
		cbBkDarkness.ItemIndex := ReadInteger('Common', 'BkDarkness', Integer(bgidWhite));

		cbAddObjectWindow.ItemIndex := ReadInteger('Common', 'AddObjectStyle', Integer(vsIcon));
		cbLinkPainter.ItemIndex := ReadInteger('Common', 'LinkPainter', 1);

		// Tree options
		checkTreeShowFields.Checked := ReadBool('TreePane', 'ShowFields', True);
		checkTreeShowObjects.Checked := ReadBool('TreePane', 'ShowObjects', True);

		editTreeFields.Text := ReadString('TreePane', 'FieldsCaption', 'Expressions');
		editTreeObjects.Text := ReadString('TreePane', 'ObjectsCaption', 'Objects');
		editTreeUnions.Text := ReadString('TreePane', 'UnionsCaption', 'Unions');

		checkExpandFields.Checked := ReadBool('TreePane', 'ExpandFields', False);
		checkExpandDatasources.Checked := ReadBool('TreePane', 'ExpandObjects', False);
		checkExpandSubqueries.Checked := ReadBool('TreePane', 'ExpandSubqueries', True);
		checkExpandUnions.Checked := ReadBool('TreePane', 'ExpandUnions', True);

		// SQL Builder options
		checkWordWrap.Checked := ReadBool('Builder', 'WordWrap', True);
		checkWordWrapClick(nil);
		editRightMargin.Value := ReadInteger('Builder', 'RightMargin', 80);

		LoadSelectFormat('MainQuery', MainSelectFormat);
		LoadSelectFormat('Subqueries', ExpressionSelectFormat);
		LoadSelectFormat('DerivedQueries', DerivedSelectFormat);

	end;
end;

procedure TfmAQBOptions.LoadSelectFormat(IniSection: String;
  SelectFormat: TSQLBuilderSelectFormat);
begin
	with Settings, SelectFormat do begin

		MainPartsFromNewLine := ReadBool(IniSection, 'MainPartNewLine', True);
		NewLineAfterPartKeywords := ReadBool(IniSection, 'KeywordsNewLine', True);
		IndentInPart := ReadInteger(IniSection, 'PartIndent', 2);
		IndentGlobal := ReadInteger(IniSection, 'GlobalIndent', 0);

		SelectListFormat.NewLineAfterItem :=
			ReadBool(IniSection, 'SelectItemNewLine', True);

		FromClauseFormat.NewLineAfterDatasource :=
			ReadBool(IniSection, 'FromObjectNewLine', False);
		FromClauseFormat.NewLineAfterJoin :=
			ReadBool(IniSection, 'FromJoinNewLine', True);

		WhereFormat.NewLineAfter := TSQLBuilderConditionFormatNL(
			ReadInteger(IniSection, 'WhereNewLine', Integer(nlAllLogical)));
		WhereFormat.IndentNestedConditions :=
			ReadInteger(IniSection, 'WhereIndent', 2);

		GroupByFormat.NewLineAfterItem :=
			ReadBool(IniSection, 'GroupItemNewLine', False);

		HavingFormat.NewLineAfter := TSQLBuilderConditionFormatNL(
			ReadInteger(IniSection, 'HavingNewLine', Integer(nlAllLogical)));
		HavingFormat.IndentNestedConditions :=
			ReadInteger(IniSection, 'HavingIndent', 2);

	end;
end;

procedure TfmAQBOptions.SaveSettingsToIni;
begin
	with Settings do begin

		// Common options
		WriteBool('Common', 'ShowTreePane', checkShowTreePane.Checked);
		WriteBool('Common', 'ShowColumnsPane', checkShowColumnsPane.Checked);
		WriteInteger('Common', 'SplitVertical', editSplitVertical.Value);
		WriteInteger('Common', 'SplitHorizontal', editSplitHorizontal.Value);

		WriteInteger('Common', 'BorderWidth', editBorderWidth.Value);
		WriteInteger('Common', 'BorderStyle', cbBorderStyle.ItemIndex);
		WriteInteger('Common', 'BkDarkness', cbBkDarkness.ItemIndex);

		WriteInteger('Common', 'AddObjectStyle', cbAddObjectWindow.ItemIndex);
		WriteInteger('Common', 'LinkPainter', cbLinkPainter.ItemIndex);

		// Tree options
		WriteBool('TreePane', 'ShowFields', checkTreeShowFields.Checked);
		WriteBool('TreePane', 'ShowObjects', checkTreeShowObjects.Checked);

		WriteString('TreePane', 'FieldsCaption', editTreeFields.Text);
		WriteString('TreePane', 'ObjectsCaption', editTreeObjects.Text);
		WriteString('TreePane', 'UnionsCaption', editTreeUnions.Text);

		WriteBool('TreePane', 'ExpandFields', checkExpandFields.Checked);
		WriteBool('TreePane', 'ExpandObjects', checkExpandDatasources.Checked);
		WriteBool('TreePane', 'ExpandSubqueries', checkExpandSubqueries.Checked);
		WriteBool('TreePane', 'ExpandUnions', checkExpandUnions.Checked);

		// SQL Builder options
		WriteBool('Builder', 'WordWrap', checkWordWrap.Checked);
		WriteInteger('Builder', 'RightMargin', editRightMargin.Value);

		SaveSelectFormat('MainQuery', MainSelectFormat);
		SaveSelectFormat('Subqueries', ExpressionSelectFormat);
		SaveSelectFormat('DerivedQueries', DerivedSelectFormat);

	end;
end;

procedure TfmAQBOptions.SaveSelectFormat(IniSection: String;
  SelectFormat: TSQLBuilderSelectFormat);
begin
	with Settings, SelectFormat do begin

		WriteBool(IniSection, 'MainPartNewLine', MainPartsFromNewLine);
		WriteBool(IniSection, 'KeywordsNewLine', NewLineAfterPartKeywords);
		WriteInteger(IniSection, 'PartIndent', IndentInPart);
		WriteInteger(IniSection, 'GlobalIndent', IndentGlobal);

		WriteBool(IniSection, 'SelectItemNewLine', SelectListFormat.NewLineAfterItem);

		WriteBool(IniSection, 'FromObjectNewLine', FromClauseFormat.NewLineAfterDatasource);
		WriteBool(IniSection, 'FromJoinNewLine', FromClauseFormat.NewLineAfterJoin);

		WriteInteger(IniSection, 'WhereNewLine', Integer(WhereFormat.NewLineAfter));
		WriteInteger(IniSection, 'WhereIndent', WhereFormat.IndentNestedConditions);

		WriteBool(IniSection, 'GroupItemNewLine', GroupByFormat.NewLineAfterItem);

		WriteInteger(IniSection, 'HavingNewLine', Integer(HavingFormat.NewLineAfter));
		WriteInteger(IniSection, 'HavingIndent', HavingFormat.IndentNestedConditions);

	end;
end;

procedure TfmAQBOptions.FormCreate(Sender: TObject);
begin
	Settings := TIniFile.Create(
		ExtractFilePath(Application.ExeName) + 'options.ini');

	ClipSelectFormat := TSQLBuilderSelectFormat.Create(nil);
	MainSelectFormat := TSQLBuilderSelectFormat.Create(nil);
	ExpressionSelectFormat := TSQLBuilderSelectFormat.Create(nil);
	DerivedSelectFormat := TSQLBuilderSelectFormat.Create(nil);

	LoadSettingsFromIni;
   pcOptions.ActivePage := tsCommon;
end;

procedure TfmAQBOptions.FormDestroy(Sender: TObject);
begin
	ClipSelectFormat.Free;
	MainSelectFormat.Free;
	ExpressionSelectFormat.Free;
	DerivedSelectFormat.Free;

	Settings.Free;
end;

procedure TfmAQBOptions.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
	Action := caHide;
end;

procedure TfmAQBOptions.FormShow(Sender: TObject);
begin
	LoadSettingsFromObject;
	DisablePasteButtons;
end;

procedure TfmAQBOptions.FormHide(Sender: TObject);
begin
	if ModalResult = mrOk
		then SaveSettingsToObject;
end;

procedure TfmAQBOptions.bSaveDefaultsClick(Sender: TObject);
begin
	SaveSettingsToIni;
end;

procedure TfmAQBOptions.checkWordWrapClick(Sender: TObject);
begin
	editRightMargin.Enabled := checkWordWrap.Checked;
end;

procedure TfmAQBOptions.sbCopyMainClick(Sender: TObject);
begin
	ClipSelectFormat.Assign(MainSelectFormat);
	EnablePasteButtons;
end;

procedure TfmAQBOptions.sbCopyExpressionClick(Sender: TObject);
begin
	ClipSelectFormat.Assign(ExpressionSelectFormat);
	EnablePasteButtons;
end;

procedure TfmAQBOptions.sbCopyDerivedClick(Sender: TObject);
begin
	ClipSelectFormat.Assign(DerivedSelectFormat);
	EnablePasteButtons;
end;

procedure TfmAQBOptions.EnablePasteButtons;
begin
	sbPasteMain.Enabled := True;
	sbPasteExpression.Enabled := True;
	sbPasteDerived.Enabled := True;
end;

procedure TfmAQBOptions.DisablePasteButtons;
begin
	sbPasteMain.Enabled := False;
	sbPasteExpression.Enabled := False;
	sbPasteDerived.Enabled := False;
end;

procedure TfmAQBOptions.sbPasteMainClick(Sender: TObject);
begin
	MainSelectFormat.Assign(ClipSelectFormat);
end;

procedure TfmAQBOptions.sbPasteExpressionClick(Sender: TObject);
begin
	ExpressionSelectFormat.Assign(ClipSelectFormat);
end;

procedure TfmAQBOptions.sbPasteDerivedClick(Sender: TObject);
begin
	DerivedSelectFormat.Assign(ClipSelectFormat);
end;

procedure TfmAQBOptions.bMainQueryClick(Sender: TObject);
begin
	fmSQLBuilderOptions.SelectFormat := MainSelectFormat;
	fmSQLBuilderOptions.ShowModal;
end;

procedure TfmAQBOptions.bExpressionSubqueryClick(Sender: TObject);
begin
	fmSQLBuilderOptions.SelectFormat := ExpressionSelectFormat;
	fmSQLBuilderOptions.ShowModal;
end;

procedure TfmAQBOptions.bDerivedSubqueryClick(Sender: TObject);
begin
	fmSQLBuilderOptions.SelectFormat := DerivedSelectFormat;
	fmSQLBuilderOptions.ShowModal;
end;

end.
